#include <ESP8266WiFi.h>
#include <WiFiClientSecure.h>
#include "DHT.h"

#define DHTTYPE DHT22 //--> Defines the type of DHT sensor used,in this project the sensor used is DHT2.

const int DHTPin = 4; //--> Pin D1 = GPIO4
DHT dht(DHTPin, DHTTYPE); 
#define ON_Board_LED 2 

//----------------------------------------SSID and Password of your WiFi router.
const char* ssid = ""; 
const char* password = ""; 

//----------------------------------------Google spreadsheet host & httpsPort
const char* host = "script.google.com";
const int httpsPort = 443;
//----------------------------------------

WiFiClientSecure client; //--> Create a WiFiClientSecure object.

String GAS_ID = "AKfycbwETCQ_Kz6KOTWnQj5y1_xczrK6FVbYuhko9UZMM_kHUblLl-lddju9wz30J5p7EkHfpg"; //--> Google spreadsheet script ID

//============================================================================== 
void setup() {

//-----Pin define for the moisture sensor
 pinMode(A0,INPUT);

  Serial.begin(115200);
  delay(500);

  dht.begin();  //--> Start reading DHT22 sensors
  delay(500);
  
  WiFi.begin(ssid, password); //--> Connect to your WiFi router
  Serial.println("");
    
  pinMode(ON_Board_LED,OUTPUT); //--> On Board LED on for board testing
  digitalWrite(ON_Board_LED, HIGH); //--> Turn off Led On Board

  //------------Wait for connection
  Serial.print("Connecting");
  while (WiFi.status() != WL_CONNECTED) {
    Serial.print(".");
    
    //----------------------------------------Make the On Board Flashing LED on the process of connecting to the wifi router.
    digitalWrite(ON_Board_LED, LOW);
    delay(250);
    digitalWrite(ON_Board_LED, HIGH);
    delay(250);
  
  }
  //----------------------------------------
  digitalWrite(ON_Board_LED, HIGH); //--> Turn off the On Board LED when it is connected to the wifi router.
 
  //----------------------------------------If successfully connected to the wifi router, the IP Address will display
  Serial.println("");
  Serial.print("Successfully connected to : ");
  Serial.println(ssid);
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());
  Serial.println();

  client.setInsecure();
}

//============================================================================== 
void loop() {
  // Reading temperature or humidity takes about 250 milliseconds!
  // Sensor readings may also be up to 2 seconds 'old' (its a very slow sensor)
  int h = dht.readHumidity();
  
  // Read temperature as Celsius 
  float t = dht.readTemperature();

  //Read moisture level 
  int m = analogRead(A0);
  
  // Check if any reads failed and exit
  if (isnan(h) || isnan(t) || isnan(m)) {
    Serial.println("Failed to read from DHT sensor !");
    delay(500);
    return;
  }
  String Temp = "Temperature : " + String(t) + " °C";
  String Humi = "Humidity : " + String(h) + " %";
  String mois = "Humidity : " + String(m) + " %";
  Serial.println(Temp);
  Serial.println(Humi);
  Serial.println(mois);
  
  sendData(t, h, m); //--> Calls the sendData Subroutine
}

//============================================================================== 
// Sending data to Google Sheets
void sendData(float tem, int hum, int mois) {
  Serial.println("==========");
  Serial.print("connecting to ");
  Serial.println(host);
  
  //----------------------------------------Connect to Google host
  if (!client.connect(host, httpsPort)) {
    Serial.println("connection failed");
    return;
  }
 

  //----------------------------------------Processing data and sending data
  String string_temperature =  String(tem);
  String string_humidity =  String(hum, DEC); 
  String string_moisture =  String(mois, DEC); 
  String url = "/macros/s/" + GAS_ID + "/exec?temperature=" + string_temperature + "&humidity=" + string_humidity + "&moisture=" + string_moisture;
  Serial.print("requesting URL: ");
  Serial.println(url);

  client.print(String("GET ") + url + " HTTP/1.1\r\n" +
         "Host: " + host + "\r\n" +
         "User-Agent: BuildFailureDetectorESP8266\r\n" +
         "Connection: close\r\n\r\n");

  Serial.println("request sent");


  //----------------------------------------Checking whether the data was sent successfully or not
  while (client.connected()) {
    String line = client.readStringUntil('\n');
    if (line == "\r") {
      Serial.println("headers received");
      break;
    }
  }
  String line = client.readStringUntil('\n');
  if (line.startsWith("{\"state\":\"success\"")) {
    Serial.println("esp8266/Arduino CI successfull!");
  } else {
    Serial.println("esp8266/Arduino CI has failed");
  }
  Serial.print("reply was : ");
  Serial.println(line);
  Serial.println("closing connection");
  Serial.println("==========");
  Serial.println();

} 
